﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading;

namespace OTDRApiClient {

    #region ServiceConfig 
    [DataContract]
    internal class ServiceConfig {
        [DataMember]
        public string pdfActiveLogo = "";
        [DataMember]
        public bool pdfDetailed = false;
        [DataMember]
        public bool pdfSummary = true;
        [DataMember]
        public string excelActiveFile = "";
        [DataMember]
        public string excelActiveLogo = "";
        [DataMember]
        public bool excelDetailed = false;
        [DataMember]
        public int excelResolution = 1;
        [DataMember]
        public bool excelHideRows = false;
    }
    #endregion

    class Program {
        static void Main(string[] args) {
            if (args.Length < 4) {
                Console.WriteLine("Use: OTDRApiClient.exe [email] [password] [process] [files]");
                Console.WriteLine("Where:");
                Console.WriteLine("[email] is a registration email");
                Console.WriteLine("[password] is a registration password");
                Console.WriteLine("[process] can be one of json, pdf, xlsx or zip");
                Console.WriteLine("[files] single file or wildcard like *.sor");
                return;
            }
            string user = args[0];       // login name (registration email), should not be empty
            string password = args[1];   // login password (registration password), should not be empty
            string culture = "en-US";    // language for results
            string units = "f";          // "m" - meters, "f" - feet
            string name = "";            // Author in PDF (unreliable)
            string process = args[2];    // "json" or "pdf" or "xlsx" or "zip"
            string in_files = args[3];   // "1.sor" or "*.sor"

            try {
                ServiceReference.SoapServiceClient client = new ServiceReference.SoapServiceClient("BasicHttpBinding_ISoapService");
                BasicHttpBinding binding = client.Endpoint.Binding as BasicHttpBinding;
                binding.MaxReceivedMessageSize = 1000000; // 1 MByte maximum file size

                string[] files = Directory.GetFiles(Environment.CurrentDirectory, in_files);
                if (files.Length > 0) {

                    #region UploadFiles
                    int i = 0;
                    string s = "";
                    bool res = true;
                    string SessionId = null;
                    string SessionTime = null;
                    foreach (string file in files) {
                        if (File.Exists(file)) {
                            MemoryStream ms = new MemoryStream(File.ReadAllBytes(file));
                            FileInfo fi = new FileInfo(file);
                            DateTime lwt = fi.LastWriteTime;

                            res &= client.UploadFiles(
                                user,                       // User
                                password,                   // Password
                                ref SessionId,              // If SessionId is null, returns SessionId, generated at SOAP Server
                                ref SessionTime,            // If SessionTime is null, returns SessionTime, generated at SOAP Server
                                culture,                    // Culture information: like "en", "en-US" etc.
                                units,                      // DisplayUnits: "m" for meters, or "f" for feets
                                file,                       // OTDR file name
                                ((lwt - new DateTime(1970, 1, 1)).Ticks / 10000000).ToString("#"), // File last write time
                                (i == files.Length - 1) ? "true" : "false",  // Last file flag 
                                null,                       // Output format: "E" for EXFO OTDR, B100 or B200 - Bellcore OTDR, or null
                                null,                       // OTDRDate or null (without DateTime modification)
                                null,                       // Randomize or null (without DateTime modification)
                                null,                       // Template File name (for xls) conversion
                                ms,                         // File content binary stream
                                out s);                     // Upload file API status
                            Console.WriteLine("Upload file " + file + ": " + s);
                            Thread.Sleep(10);
                            i++;
                        } else {
                            Console.WriteLine("File " + file + " not found");
                            res = false;
                        }
                    }
                    #endregion

                    if (res) {

                        #region RefreshFiles 
                        Console.WriteLine("Wait for result");

                        #region Service Config
                        ServiceConfig config = new ServiceConfig();
                        config.excelActiveFile = "1.xlxt";
                        config.excelHideRows = true;
                        config.pdfSummary = true;
                        DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(ServiceConfig));
                        MemoryStream ms = new MemoryStream();
                        ser.WriteObject(ms, config);
                        byte[] json = ms.ToArray();
                        ms.Close();
                        #endregion

                        bool done = false;
                        for (i = 0; i < 10000; i++) {
                            Thread.Sleep(1000);
                            res = client.RefreshFiles(
                                user,                       // User
                                password,                   // Password
                                ref SessionId,              // If SessionId is null, returns SessionId, generated at SOAP Server
                                ref SessionTime,            // If SessionTime is null, returns SessionTime, generated at SOAP Server
                                culture,                    // Culture information: like "en", "en-US" etc.
                                process,                    // Process: "json" or "pdf" or "xlsx" or "zip"
                                Encoding.UTF8.GetString(json), // Service Config
                                units,                      // DisplayUnits: "m" for meters, or "f" for feets
                                name,                       // Operator name (used in PDF properties)
                                out s);                     // Refresh file API status                        
                            if (res) {
                                Console.WriteLine("Refresh done");
                                done = true;
                                break;
                            } else {
                                if (s != "processing") {
                                    Console.WriteLine(s);
                                    break;
                                }
                            }
                        }
                        #endregion

                        #region Download
                        if (done == true) {
                            byte[] stream = null;
                            res = client.DownloadFile(
                                user,           // User
                                password,       // Password
                                SessionId,      // If SessionId is null, returns SessionId, generated at SOAP Server
                                SessionTime,    // If SessionTime is null, returns SessionTime, generated at SOAP Server
                                culture,        // Culture information: like "en", "en-US" etc.
                                process,        // Process: "json" or "pdf" or "xlsx" or "zip", should be the same as in "RefreshFiles"
                                out s,          // State (error) string
                                out stream);    // Output stream
                            if (res) {
                                if (stream != null && stream.Length > 0) {
                                    File.WriteAllBytes("Result." + process, stream);
                                    Console.WriteLine("Download done. File " + Environment.CurrentDirectory + Path.DirectorySeparatorChar + "Result." + process);
                                } else {
                                    Console.WriteLine("Stream error");
                                }
                            } else {
                                Console.WriteLine("Download failed (" + s + ")");
                            }
                        }
                        #endregion

                        #region DeleteFiles
                        res = client.DeleteFiles(
                                user,               // User
                                password,           // Password
                                ref SessionId,      // If SessionId is null, returns SessionId, generated at SOAP Server
                                ref SessionTime,    // If SessionTime is null, returns SessionTime, generated at SOAP Server
                                culture,            // Culture information: like "en", "en-US" etc.
                                out s);             // State (error) string
                        if (res) {
                            Console.WriteLine("Delete server session files success");
                        } else Console.WriteLine("Delete server session files failed (" + s + ")");
                        #endregion

                    } else { Console.WriteLine("Upload failed"); }
                } else { Console.WriteLine("No files found"); }

                client.Close();
            } catch (Exception ex) { Console.WriteLine(ex); }

            Console.WriteLine("Finished. Press ENTER");
            Console.ReadLine();
        }
    }
}
